#!/usr/bin/env python3
"""
Encryption Analyzer
Advanced encryption analysis and security assessment tool.
Parameters:
    - target: Target file or directory to analyze
    - mode: Analysis mode (file, directory, memory)
    - output: Output file for results
Requires: cryptography, pycryptodome packages
"""
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
from collections import defaultdict
import os
import math
import binascii
import re
import magic
import mmap
import hashlib
from Crypto.Cipher import AES, DES, DES3, Blowfish, ARC2, ARC4
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import serialization
from cryptography.x509 import load_pem_x509_certificate
import yara

class EncryptionAnalyzer:
    def __init__(self, target, mode='file'):
        self.target = target
        self.mode = mode.lower()
        self.results = defaultdict(dict)
        self.setup_logging()
        self.load_rules()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def load_rules(self):
        """Load YARA rules for encryption detection"""
        try:
            rules_text = """
            rule Encryption_Artifacts {
                strings:
                    $aes1 = "AES" nocase
                    $aes2 = "Rijndael" nocase
                    $des1 = "DES" nocase
                    $des2 = "3DES" nocase
                    $rsa1 = "RSA" nocase
                    $ssl1 = "SSL" nocase
                    $tls1 = "TLS" nocase
                    $pgp1 = "PGP" nocase
                    $gpg1 = "GPG" nocase
                    $blowfish = "Blowfish" nocase
                    $rc4 = "RC4" nocase
                    $sha1 = "SHA" nocase
                    $md5 = "MD5" nocase
                condition:
                    any of them
            }

            rule Encryption_Functions {
                strings:
                    $enc1 = "encrypt" nocase
                    $enc2 = "decrypt" nocase
                    $enc3 = "cipher" nocase
                    $enc4 = "aes_encrypt" nocase
                    $enc5 = "des_encrypt" nocase
                    $enc6 = "rsa_encrypt" nocase
                condition:
                    any of them
            }

            rule Encryption_Keys {
                strings:
                    $key1 = "private key" nocase
                    $key2 = "public key" nocase
                    $key3 = "secret key" nocase
                    $key4 = "encryption key" nocase
                    $key5 = "BEGIN PRIVATE KEY" nocase
                    $key6 = "BEGIN PUBLIC KEY" nocase
                    $key7 = "BEGIN RSA PRIVATE KEY" nocase
                    $key8 = "BEGIN CERTIFICATE" nocase
                condition:
                    any of them
            }
            """
            self.rules = yara.compile(source=rules_text)
        except Exception as e:
            self.logger.error(f"Error loading YARA rules: {str(e)}")
            self.rules = None

    def calculate_entropy(self, data):
        """Calculate Shannon entropy of data"""
        if not data:
            return 0
        
        entropy = 0
        for x in range(256):
            p_x = float(data.count(x))/len(data)
            if p_x > 0:
                entropy += - p_x*math.log(p_x, 2)
        return entropy

    def detect_encryption_type(self, data):
        """Attempt to detect encryption type from data patterns"""
        patterns = {
            'AES': [
                rb'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
            ],
            'DES': [
                rb'\x01\x23\x45\x67\x89\xab\xcd\xef'
            ],
            '3DES': [
                rb'\x01\x23\x45\x67\x89\xab\xcd\xef\x01\x23\x45\x67\x89\xab\xcd\xef'
            ],
            'RSA': [
                rb'-----BEGIN RSA PRIVATE KEY-----',
                rb'-----BEGIN PUBLIC KEY-----'
            ],
            'PGP': [
                rb'-----BEGIN PGP MESSAGE-----',
                rb'-----BEGIN PGP PUBLIC KEY BLOCK-----'
            ],
            'SSL/TLS': [
                rb'-----BEGIN CERTIFICATE-----',
                rb'-----BEGIN PRIVATE KEY-----'
            ]
        }

        detected = []
        for enc_type, patterns_list in patterns.items():
            for pattern in patterns_list:
                if pattern in data:
                    detected.append(enc_type)
                    break

        return detected

    def analyze_file_entropy(self, file_path):
        """Analyze file entropy and detect potential encryption"""
        try:
            with open(file_path, 'rb') as f:
                data = f.read()
                
            file_info = {
                'path': file_path,
                'size': len(data),
                'entropy': self.calculate_entropy(data),
                'mime_type': magic.from_file(file_path, mime=True),
                'encryption_detected': False,
                'encryption_types': [],
                'key_material': False,
                'hashes': {
                    'md5': hashlib.md5(data).hexdigest(),
                    'sha256': hashlib.sha256(data).hexdigest()
                },
                'yara_matches': []
            }

            # Check entropy
            if file_info['entropy'] > 7.0:
                file_info['encryption_detected'] = True

            # Detect encryption types
            file_info['encryption_types'] = self.detect_encryption_type(data)
            if file_info['encryption_types']:
                file_info['encryption_detected'] = True

            # YARA scanning
            if self.rules:
                matches = self.rules.match(data=data)
                if matches:
                    file_info['yara_matches'] = [match.rule for match in matches]
                    if 'Encryption_Keys' in file_info['yara_matches']:
                        file_info['key_material'] = True

            return file_info

        except Exception as e:
            self.logger.error(f"Error analyzing {file_path}: {str(e)}")
            return None

    def analyze_memory_region(self, pid, region):
        """Analyze a memory region for encryption artifacts"""
        try:
            with open(f"/proc/{pid}/mem", 'rb') as f:
                f.seek(region['start'])
                data = f.read(region['end'] - region['start'])

            region_info = {
                'pid': pid,
                'start': region['start'],
                'end': region['end'],
                'size': len(data),
                'entropy': self.calculate_entropy(data),
                'encryption_detected': False,
                'encryption_types': [],
                'key_material': False,
                'yara_matches': []
            }

            # Check entropy
            if region_info['entropy'] > 7.0:
                region_info['encryption_detected'] = True

            # Detect encryption types
            region_info['encryption_types'] = self.detect_encryption_type(data)
            if region_info['encryption_types']:
                region_info['encryption_detected'] = True

            # YARA scanning
            if self.rules:
                matches = self.rules.match(data=data)
                if matches:
                    region_info['yara_matches'] = [match.rule for match in matches]
                    if 'Encryption_Keys' in region_info['yara_matches']:
                        region_info['key_material'] = True

            return region_info

        except Exception as e:
            self.logger.error(
                f"Error analyzing memory region {region['start']}-{region['end']}: {str(e)}"
            )
            return None

    def analyze_certificate(self, cert_data):
        """Analyze SSL/TLS certificate"""
        try:
            cert = load_pem_x509_certificate(cert_data)
            return {
                'subject': str(cert.subject),
                'issuer': str(cert.issuer),
                'valid_from': cert.not_valid_before.isoformat(),
                'valid_until': cert.not_valid_after.isoformat(),
                'serial_number': cert.serial_number,
                'version': cert.version,
                'signature_algorithm': cert.signature_algorithm_oid._name,
                'key_size': cert.public_key().key_size,
                'issues': []
            }
        except Exception as e:
            return {
                'error': str(e),
                'issues': ['Invalid certificate format']
            }

    def analyze_key_strength(self, key_data):
        """Analyze cryptographic key strength"""
        try:
            # Try to load as RSA key
            if b'BEGIN RSA PRIVATE KEY' in key_data:
                key = serialization.load_pem_private_key(
                    key_data,
                    password=None
                )
                if isinstance(key, rsa.RSAPrivateKey):
                    return {
                        'type': 'RSA',
                        'key_size': key.key_size,
                        'public_numbers': {
                            'e': key.public_key().public_numbers().e,
                            'n': key.public_key().public_numbers().n
                        },
                        'issues': []
                    }
            return None
        except Exception as e:
            return {
                'error': str(e),
                'issues': ['Invalid key format']
            }

    def analyze_directory(self, directory):
        """Recursively analyze directory for encryption artifacts"""
        for root, _, files in os.walk(directory):
            for file in files:
                file_path = os.path.join(root, file)
                result = self.analyze_file_entropy(file_path)
                if result and (
                    result['encryption_detected'] or
                    result['key_material']
                ):
                    self.results['files'][file_path] = result

    def analyze_process_memory(self, pid):
        """Analyze process memory for encryption artifacts"""
        try:
            with open(f"/proc/{pid}/maps", 'r') as f:
                for line in f:
                    fields = line.split()
                    if len(fields) >= 5:
                        addr_range = fields[0]
                        perms = fields[1]
                        
                        if 'r' in perms:  # Only analyze readable regions
                            start, end = map(
                                lambda x: int(x, 16),
                                addr_range.split('-')
                            )
                            region = {'start': start, 'end': end}
                            result = self.analyze_memory_region(pid, region)
                            if result and (
                                result['encryption_detected'] or
                                result['key_material']
                            ):
                                self.results['memory'][f"{pid}:{addr_range}"] = result

        except Exception as e:
            self.logger.error(f"Error analyzing process {pid}: {str(e)}")

    def save_results(self, output_file):
        """Save analysis results to file"""
        scan_results = {
            'timestamp': datetime.now().isoformat(),
            'target': self.target,
            'mode': self.mode,
            'results': dict(self.results)
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(scan_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        """Print analysis summary"""
        self.logger.info("\nEncryption Analysis Summary:")
        
        if 'files' in self.results:
            encrypted_files = len([
                f for f in self.results['files'].values()
                if f['encryption_detected']
            ])
            key_material_files = len([
                f for f in self.results['files'].values()
                if f['key_material']
            ])
            
            self.logger.info("\nFile Analysis:")
            self.logger.info(f"Files analyzed: {len(self.results['files'])}")
            self.logger.info(f"Encrypted files detected: {encrypted_files}")
            self.logger.info(f"Files with key material: {key_material_files}")
            
            # Show high-entropy files
            high_entropy_files = [
                f for f in self.results['files'].items()
                if f[1]['entropy'] > 7.0
            ]
            if high_entropy_files:
                self.logger.info("\nHigh Entropy Files:")
                for path, info in high_entropy_files:
                    self.logger.info(
                        f"\nFile: {path}\n"
                        f"Entropy: {info['entropy']:.2f}\n"
                        f"Size: {info['size']} bytes\n"
                        f"Encryption Types: {', '.join(info['encryption_types'])}"
                    )

        if 'memory' in self.results:
            encrypted_regions = len([
                r for r in self.results['memory'].values()
                if r['encryption_detected']
            ])
            key_material_regions = len([
                r for r in self.results['memory'].values()
                if r['key_material']
            ])
            
            self.logger.info("\nMemory Analysis:")
            self.logger.info(
                f"Memory regions analyzed: {len(self.results['memory'])}"
            )
            self.logger.info(f"Encrypted regions detected: {encrypted_regions}")
            self.logger.info(f"Regions with key material: {key_material_regions}")
            
            # Show regions with key material
            if key_material_regions > 0:
                self.logger.info("\nRegions with Key Material:")
                for region, info in self.results['memory'].items():
                    if info['key_material']:
                        self.logger.info(
                            f"\nRegion: {region}\n"
                            f"Size: {info['size']} bytes\n"
                            f"Encryption Types: {', '.join(info['encryption_types'])}"
                        )

    def start_analysis(self, output_file=None):
        """Start encryption analysis"""
        self.logger.info(f"Starting encryption analysis")
        self.logger.info(f"Target: {self.target}")
        self.logger.info(f"Mode: {self.mode}")
        
        try:
            if self.mode == 'file':
                result = self.analyze_file_entropy(self.target)
                if result:
                    self.results['files'][self.target] = result
                    
            elif self.mode == 'directory':
                self.analyze_directory(self.target)
                
            elif self.mode == 'memory':
                try:
                    pid = int(self.target)
                    self.analyze_process_memory(pid)
                except ValueError:
                    self.logger.error("Invalid PID specified for memory analysis")
                
        except KeyboardInterrupt:
            self.logger.info("Analysis interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during analysis: {str(e)}")
        finally:
            if output_file:
                self.save_results(output_file)
            self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Encryption Analyzer")
    parser.add_argument("target", help="Target file or directory to analyze")
    parser.add_argument("--mode", choices=['file', 'directory', 'memory'],
                      default='file', help="Analysis mode")
    parser.add_argument("--output", help="Output file for analysis results")
    args = parser.parse_args()

    analyzer = EncryptionAnalyzer(args.target, args.mode)
    analyzer.start_analysis(args.output)
